import { createQuery } from '@tanstack/svelte-query';
import { writable } from 'svelte/store';
import { buildLoadingQueryObserver } from './QueryObserver.js';
/**
 * Uses the Svelte Query library to fetch your paginated table data.
 */
export class SvelteQueryDataSource {
    apiFunction;
    additionalQueryOptions;
    dataQuery;
    dataQueryUnsubscribe;
    dataObserver;
    initialized = false;
    queryKeyPrefix;
    queryKey = [];
    queryEnabled = undefined;
    /**
     * Create a new data source to fetch your paginated table data from an api endpoint
     * using Svelte Query in the background to manage requests, caching and more.
     *
     * @param apiFunction the api function which actually fetches your data, passed through to Svelte Query as the [query function](https://sveltequery.vercel.app/guides/query-functions).
     * A query function can be literally any function that returns a promise. The promise that is returned should either resolve the data or throw an error.
     * @param additionalQueryOptions additional options passed to Svelte Query
     *
     * @see {@link https://sveltequery.vercel.app/reference/useQuery}
     */
    constructor(apiFunction, additionalQueryOptions = {
        keepPreviousData: true
    }) {
        this.apiFunction = apiFunction;
        this.additionalQueryOptions = additionalQueryOptions;
        this.dataObserver = writable(buildLoadingQueryObserver());
        if (typeof this.additionalQueryOptions.enabled !== 'undefined') {
            this.queryEnabled = !!this.additionalQueryOptions.enabled;
        }
    }
    getQueryObserver() {
        return this.dataObserver;
    }
    init(config) {
        this.queryKeyPrefix = `dataTable-${config.type}`;
        this.initialized = true;
    }
    requestData(data) {
        if (!this.initialized) {
            throw new Error('Svelte-Query data source was not properly initialized before requesting data');
        }
        this.queryEnabled ??= true;
        this.queryKey = [this.queryKeyPrefix, data];
        this.updateQueryOptions({});
    }
    setApiFunction(apiFunction) {
        this.apiFunction = apiFunction;
        this.updateQueryOptions({});
    }
    setEnabled(enabled) {
        this.queryEnabled = enabled;
        this.updateQueryOptions({});
    }
    updateQueryOptions(queryOptions) {
        if (typeof queryOptions.enabled !== 'undefined') {
            this.queryEnabled = !!queryOptions.enabled;
        }
        this.additionalQueryOptions = {
            ...this.additionalQueryOptions,
            ...queryOptions
        };
        if (!this.initialized) {
            return;
        }
        if (this.dataQueryUnsubscribe) {
            this.dataQueryUnsubscribe();
        }
        console.trace('Updating query with', {
            ...this.additionalQueryOptions,
            queryKey: this.queryKey,
            queryFn: this.wrapApiFunction(),
            enabled: this.queryEnabled ?? false
        });
        this.dataQuery = createQuery({
            ...this.additionalQueryOptions,
            queryKey: this.queryKey,
            queryFn: this.wrapApiFunction(),
            enabled: this.queryEnabled ?? false
        });
        this.dataQueryUnsubscribe = this.dataQuery.subscribe((value) => this.dataObserver.set(value));
    }
    wrapApiFunction() {
        return ({ queryKey }) => {
            return this.apiFunction(queryKey[1]);
        };
    }
}
